'''
Created on April 28, 2020
@author: Andrew Habib
'''

import sys
from collections import Counter, OrderedDict

path_to_ground_truth = 'table4-ground-truth.txt'


def load_results(path):
    with open(path, 'r') as fh:
        lines = fh.readlines()
        lines = lines[:300]  # trim extra line printed from shell scripts

    everything = []
    for line in lines:
        line = line.strip('\n').strip(' ')
        if line != 'True' and line != 'False':
            line = 'UNK'
        everything.append(line)

    snowplow = everything[:50]
    wp = everything[50:100]
    k8 = everything[100:150]
    lale = everything[150:200]
    mix = everything[200:]

    return snowplow, wp, k8, lale, mix, everything


def calculate_metrics(ground_truths, results):
    metrics = OrderedDict(
        {'TP': 0, 'TN': 0, 'FP': 0, 'FN': 0, 'TrueUNK': 0, 'FalseUNK': 0})

    for t, r in zip(ground_truths, results):
        if t == r == 'True':
            metrics['TP'] = metrics['TP'] + 1
        elif t == r == 'False':
            metrics['TN'] = metrics['TN'] + 1
        elif t == 'False' and r == 'True':
            metrics['FP'] = metrics['FP'] + 1
        elif t == 'True' and r == 'False':
            metrics['FN'] = metrics['FN'] + 1
        elif t == 'True' and r == 'UNK':
            metrics['TrueUNK'] = metrics['TrueUNK'] + 1
        elif t == 'False' and r == 'UNK':
            metrics['FalseUNK'] = metrics['FalseUNK'] + 1

    precision = 100 * metrics['TP'] / (metrics['TP'] + metrics['FP'])
    recall = 100 * metrics['TP'] / \
        (metrics['TP'] + metrics['FN'] + metrics['TrueUNK'])
    correctness = 100 * (metrics['TP'] + metrics['TN']) / \
        (metrics['TP'] + metrics['TN'] + metrics['FP'] + metrics['FN'])

    return metrics, precision, recall, correctness


def main():

    path_to_tool_results = sys.argv[1]

    snowplow_t, wp_t, k8_t, lale_t, mix_t, everything_t = load_results(
        path_to_ground_truth)
    snowplow_t_counter, wp_t_counter, k8_t_counter, lale_t_counter, mix_t_counter, everything_t_counter = Counter(
        snowplow_t), Counter(wp_t), Counter(k8_t), Counter(lale_t), Counter(mix_t), Counter(everything_t)

    snowplow, wp, k8, lale, mix, everything = load_results(
        path_to_tool_results)
    snowplow_counter, wp_counter, k8_counter, lale_counter, mix_counter, everything_counter = Counter(
        snowplow), Counter(wp), Counter(k8), Counter(lale), Counter(mix), Counter(everything)

    snowplow_metrics, _, _, _ = calculate_metrics(snowplow_t, snowplow)
    wp_metrics, _, _, _ = calculate_metrics(wp_t, wp)
    k8_metrics, _, _, _ = calculate_metrics(k8_t, k8)
    lale_metrics, _, _, _ = calculate_metrics(lale_t, lale)
    mix_metrics, _, _, _ = calculate_metrics(mix_t, mix)
    everything_metrics, precision, recall, correctness = calculate_metrics(
        everything_t, everything)

    print()
    print(f'                Ground truth   |           Tool')
    print('            -----------------------------------------------')
    print(f'            Pairs    <:  </:   |  TP  TN  FP  FN  <:?  </:?')
    print('            -----------------------------------------------')
    print(
        f'   Snowplow   {len(snowplow_t):3}   {snowplow_t_counter["True"]:3}  {snowplow_t_counter["False"]:3}   | {snowplow_metrics["TP"]:3} {snowplow_metrics["TN"]:3} {snowplow_metrics["FP"]:3} {snowplow_metrics["FN"]:3}  {snowplow_metrics["TrueUNK"]:3}   {snowplow_metrics["FalseUNK"]:3}')
    print(
        f'         WP   {len(wp_t):3}   {wp_t_counter["True"]:3}  {wp_t_counter["False"]:3}   | {wp_metrics["TP"]:3} {wp_metrics["TN"]:3} {wp_metrics["FP"]:3} {wp_metrics["FN"]:3}  {wp_metrics["TrueUNK"]:3}   {wp_metrics["FalseUNK"]:3}')
    print(
        f'         K8   {len(k8_t):3}   {k8_t_counter["True"]:3}  {k8_t_counter["False"]:3}   | {k8_metrics["TP"]:3} {k8_metrics["TN"]:3} {k8_metrics["FP"]:3} {k8_metrics["FN"]:3}  {k8_metrics["TrueUNK"]:3}   {k8_metrics["FalseUNK"]:3}')
    print(
        f'       Lale   {len(lale_t):3}   {lale_t_counter["True"]:3}  {lale_t_counter["False"]:3}   | {lale_metrics["TP"]:3} {lale_metrics["TN"]:3} {lale_metrics["FP"]:3} {lale_metrics["FN"]:3}  {lale_metrics["TrueUNK"]:3}   {lale_metrics["FalseUNK"]:3}')
    print('-----------------------------------------------------------')
    print(
        f'  Unrelated   {len(mix_t):3}   {mix_t_counter["True"]:3}  {mix_t_counter["False"]:3}   | {mix_metrics["TP"]:3} {mix_metrics["TN"]:3} {mix_metrics["FP"]:3} {mix_metrics["FN"]:3}  {mix_metrics["TrueUNK"]:3}   {mix_metrics["FalseUNK"]:3}')
    print('-----------------------------------------------------------')
    print(
        f'      Total   {len(everything):3}   {everything_t_counter["True"]:3}  {everything_t_counter["False"]:3}   | {everything_metrics["TP"]:3} {everything_metrics["TN"]:3} {everything_metrics["FP"]:3} {everything_metrics["FN"]:3}  {everything_metrics["TrueUNK"]:3}   {everything_metrics["FalseUNK"]:3}')
    print('-----------------------------------------------------------')
    print(f'  Precision                    |           {precision:6.2f}')
    print(f'     Recall                    |           {recall:6.2f}')
    print(f'Correctness                    |           {correctness:6.2f}')
    print()


if __name__ == "__main__":

    main()
