'''
Created on Jan. 27, 2021
@author: Andrew Habib
'''

import json
import os
import time
from jsonsubschema import isSubschema
import matplotlib.pyplot as plt

path_to_list_of_schemas = 'figure6-schemas.txt'
path_to_raw_data = 'size_vs_time_raw_data.txt'
path_to_plot = 'size_vs_time.eps'

data_points_count = 1000
reps = 10


def measure_performance():
    print("Measuring performance...")
    with open(path_to_list_of_schemas, 'r') as fh:
        list_of_schema_pairs = fh.readlines()

    results = []

    for l in list_of_schema_pairs:
        lhs, rhs = l.split()
        try:
            with open(lhs) as f:
                lhs_schema = json.load(f)
            with open(rhs) as f:
                rhs_schema = json.load(f)
            times = []

            for i in range(reps):
                time_start = time.time()
                isSubschema(lhs_schema, rhs_schema)
                time_end = time.time()
                times.append(time_end - time_start)
            avg_time = sum(times)/len(times)
            lhs_size, rhs_size = os.path.getsize(lhs), os.path.getsize(rhs)
            results.append(((lhs_size+rhs_size), avg_time))
            if len(results) == data_points_count:
                break
        except Exception as e:
            continue

    print("Collecting data points is done.")
    print(f'Saving results in {path_to_raw_data}')
    with open(path_to_raw_data, 'w') as fh:
        for s, t in results:
            fh.write(f'{s} {t}\n')


def plot_results():
    print(f'Plotting results in {path_to_plot}')
    size_to_time = []
    with open(path_to_raw_data, 'r') as fh:
        for l in fh.readlines():
            size, t = l.split()
            size_to_time.append((float(size)/1024, float(t)))
    size_to_time = sorted(size_to_time)
    size_to_time = zip(*size_to_time)
    sizes, times = [list(tuple) for tuple in size_to_time]

    plt.grid()
    plt.scatter(sizes, times)
    plt.xlabel("Size of files of pairs of schemas in KB")
    plt.ylabel("Runtime in seconds")
    plt.savefig(path_to_plot, format="eps")


def main():
    measure_performance()
    plot_results()


if __name__ == "__main__":

    main()
