'''
Created on April 28, 2020
@author: Andrew Habib
'''

import json
import jsonschema
import os
import unittest

import jsonref
from jsonsubschema._canonicalization import (
    canonicalize_schema,
    simplify_schema_and_embed_checkers
)

TESTSSUITE_PATH = "draft4"

if __name__ == "__main__":

    count_tests, count_schemas = 0, 0
    passed_tests, failed_canonicalization, failed_tests, failed_corner, missed_tests = 0, 0, 0, 0, 0
    for dirName, subdirList, fileList in os.walk(TESTSSUITE_PATH):
        if dirName == 'draft4/optional':
            # Skipping optional tests as they already cause some troubles to the json schema validators itself.
            continue

        for f in fileList:
            if f == "format.json":
                # Skipping format tests as the format keyword is not part of the validation keywords.
                # One can, however, improve our subschema checker by handling the format keyword too.
                continue

            tests_in_file = json.load(open(dirName + "/" + f, "r"))
            for test in tests_in_file:
                schema = test["schema"]
                count_schemas += 1
                count_tests += len(test["tests"])
                try:
                    ''' This outer try catches failures in canonicalization '''
                    normalized_schema = jsonref.JsonRef.replace_refs(schema)
                    normalized_schema = simplify_schema_and_embed_checkers(
                        canonicalize_schema(normalized_schema))

                    for t in test["tests"]:
                        expected = t["valid"]
                        data = t["data"]
                        try:
                            validator = jsonschema.Draft4Validator(
                                normalized_schema)
                            if validator.is_valid(data) == expected:
                                passed_tests += 1
                            else:
                                failed_tests += 1

                        except Exception as e:
                            failed_corner += 1

                except Exception as e:
                    failed_canonicalization += 1
                    missed_tests += len(test["tests"])

    print("TOTAL # of schemas:", count_schemas)
    print("TOTAL # of tests:", count_tests)
    print("TOTAL # of passed tests:", passed_tests)
    print("TOTAL # of failed tests", failed_tests + failed_corner)

    print()
    print("TOTAL # of schemas which jsonsubschema could not canonicalize",
          failed_canonicalization)
    print("TOTAL # of skipped tests due to un-canonicalized schemas", missed_tests)
